# Copyright (c) 2021-2023 Alibaba Cloud Computing Ltd.
# SPDX-License-Identifier: MulanPSL-2.0
#!/usr/bin/python3
#-*- coding = utf-8 -*-

import re
import sys
import subprocess
import logging
import paramiko
import time


"""
memtier_benchmark 
A memcache/redis NoSQL traffic generator and performance benchmarking tool.
"""

logger = logging.getLogger(__name__)

# const
DEFAULT_TIME = 60
DEFAULT_THREADS = 8
DEFAULT_PORT = 9400
DEFAULT_CLIENT = 10
DEFAULT_RATIO = "1:10"
DEFAULT_PIPE = 1
DEFAULT_DATASIZE = 32
DEFAULT_MAXKEY = 100000
PORT = 22
USER = 'root'
HOSTPWD = ''


class Benchmark():
    def __init__(self, url, times=DEFAULT_TIME, threads=DEFAULT_THREADS, port=DEFAULT_PORT, clients=DEFAULT_CLIENT, ratio=DEFAULT_RATIO, pipe=DEFAULT_PIPE, datasize=DEFAULT_DATASIZE, keymax=DEFAULT_MAXKEY, hport=PORT, huser=USER, hpwd=HOSTPWD):
        """Init benchmark

        Args:
            url (string): url
            connections (int, optional): Connections to keep open. Defaults to 300.
            threads (int, optional):  Number of threads to use. Defaults to 10.
            duration (int, optional): Duration of test. Defaults to 30.
        """
        self.url = url
        self.times = times
        self.threads = threads
        self.port = port
        self.client = clients
        self.ratio = ratio
        self.pipe = pipe
        self.datasize = datasize
        self.keymax = keymax
        self.hport = hport
        self.huser = huser
        self.hpwd = hpwd

    def __transfMeasurement(self,value,measurement):
        if measurement == '':
            return value

        # measurement of Latency
        elif measurement == 'h':
            return value * 60 * 60 * 10 ** 6
        elif measurement == 'm':
            return value * 60 * 10 ** 6
        elif measurement == "s":
            return value * 10 ** 6
        elif measurement == "ms":
            return value * 10 ** 3
        elif measurement == 'us':
            return value

        # measurement of Req/Sec
        elif measurement == "k":
            return value * 10 ** 3
        elif measurement == 'M':
            return value * 10 ** 6
        elif measurement == 'G':
            return value * 10 ** 9

        # measurement of Transfer/sec
        elif measurement == "KB":
            return value * 10 ** 3
        elif measurement == 'MB':
            return value * 10 ** 6
        elif measurement == 'GB':
            return value * 10 ** 9

        else:
            logger.warning("Unknown measurement: {}".format(measurement))
            return value
    def conf_redis(self):
        print("deploy rediserver")
        ssh = paramiko.SSHClient()
        key = paramiko.AutoAddPolicy()
        ssh.set_missing_host_key_policy(key)
        ssh.connect(self.url,self.hport,self.huser,self.hpwd,timeout=60)
        _, stdout, stderr = ssh.exec_command('sh /tmp/conf.sh')
        if 'ok' in str(stdout.read().decode('utf-8')):
            stat = True
            ssh.exec_command('redis-cli -p{}'.format(self.hport))
        else:
            stat = False
        time.sleep(5)
        ssh.close()
        print("redis config complate")
        return stat

    def run(self):
        """Run benchmark and parse output

        Return True and score list if running benchmark successfully, otherwise return False and empty list.
        """
        if self.conf_redis():
            stat = True
        else:
            logger.error("conf redis failed")
            return False, []
        if stat:
            cmd = 'memtier_benchmark -s {} -p {} -t {} --test-time={}  -c {} --ratio={} --pipeline={} -d {} --key-maximum={}'.format(
                    self.url,self.port,self.threads,self.times,self.client,self.ratio,self.pipe,self.datasize,self.keymax)
            logger.info(cmd)
            result = subprocess.run(
                        cmd,
                        shell=True,
                        close_fds=True,
                        stderr=subprocess.PIPE,
                        stdout=subprocess.PIPE
                    )
            self.out = result.stdout.decode('UTF-8','strict')
            self.error = result.stderr.decode('UTF-8','strict')
            if result.returncode == 0:
                logger.info(self.out)

                pattern_ops_latency_requests = re.compile(r"Totals\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)\s+([\d\.]+)")
                if not re.search(pattern_ops_latency_requests,self.out):
                    logger.error("can not parse output: {}".format(self.out))
                    return False, []

                options = float(re.search(pattern_ops_latency_requests,self.out).group(1))
                hits = float(re.search(pattern_ops_latency_requests,self.out).group(2))
                latency = float(re.search(pattern_ops_latency_requests,self.out).group(4))
                throuput = float(re.search(pattern_ops_latency_requests,self.out).group(5))

                result = {
                    "OPS": options,
                    "HPS": hits,
                    "Latency": latency,
                    "Throuput": throuput,
                }

                result_str = ", ".join(["{} = {}".format(k,v) for k,v in result.items()])
                print(result_str)
                return True, result_str

            else:
                logger.error(self.error)
                return False, []
        else:
            logger.error("conf redis failed")
            return False, []

if __name__ == "__main__":
    if sys.argv.__len__() <= 1:
        print("'Target ip' is wanted: python3 ack_nginx_http_short.py [Target ip]")
        exit(1)
    bench = Benchmark(sys.argv[1])
    suc, res = bench.run()
    print("The status is:", suc)
    print("The result is:", res)

